/*
 * Copyright 2008 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 */


/*
 * DirectionJudgeImpl.hpp
 *
 *  Created on: 2011/01/14
 *      Author: HARK Development Team
 */

#ifndef DIRECTIONJUDGEIMPL_HPP_
#define DIRECTIONJUDGEIMPL_HPP_

#include <vector>
#include <boost/shared_ptr.hpp>
#include <DirectionJudge.hpp>
#include <DimensionPromise/types/Length.hpp>
#include <tickdegree.hpp>

namespace NSHarkBinaural
{

class JudgeEpipolar : public DirectionJudge
{
public:
    JudgeEpipolar(NSDimensionPromise::Length sourceDistance, NSDimensionPromise::Length headRadius);

    virtual DirectionProbabilityVectorPtr
    Judge(const std::pair<BandHarmonicVectorPtr, BandHarmonicVectorPtr>& dividedBands,
          IPD::ptr_t ipd, IID::ptr_t iid, MethodFlag::Kind mflag, bool sortByProbability);

private:
    std::vector<Probability> CalcProbabilityFromIPD(const BandHarmonicVectorPtr ipdh, IPD::ptr_t ipd);
    std::vector<Probability> CalcProbabilityFromIID(const BandHarmonicVectorPtr iidh, IID::ptr_t iid);
    //static AngleAndProbability MakeAngleAndProbability(const std::pair<size_t, Probability>& indexedProbability);

private:
    NSDimensionPromise::Length sourceDistance;
    NSDimensionPromise::Length headRadius;

    /**
     * 信号源から両耳までの距離差モデルの配列.
     * 各角度に対する、理論上の距離差。
     * 最初の要素が左耳(+90度)、最後の要素が右耳(-90度)、中間の要素が正面(0度)に対応する。
     * すなわち、i * TICKDEGREE度の情報は
     * lrLengthDifference[180 / TICKDEGREE / 2 - i]に格納される。
     */
    std::vector<NSDimensionPromise::Length> lrLengthDifference;
};

class IIPDS;

/**
 * 測定した頭部伝達関数を利用して角度の識別を行う.
 */
class JudgeHRTFMeasured : public DirectionJudge
{
public:
    JudgeHRTFMeasured(boost::shared_ptr<IIPDS> iipds);

    virtual DirectionProbabilityVectorPtr
    Judge(const std::pair<BandHarmonicVectorPtr, BandHarmonicVectorPtr>& dividedBands,
          IPD::ptr_t ipd, IID::ptr_t iid, MethodFlag::Kind mflag, bool sortByProbability);

private:
    //AngleAndProbability MakeAngleAndProbability(const std::pair<size_t, Probability>& indexedProbability);

private:
    boost::shared_ptr<IIPDS> iipds;
};

}

#endif /* DIRECTIONJUDGEIMPL_HPP_ */
